#!/usr/bin/perl 

use Getopt::Std;
require "read_config.pl";


getopts("hSDHWAP");

$debug=1;
$home=".";

if($opt_h){
    help();
    exit;
}


my $params_cache="params.cache";

if( -f $params_cache and !-z $params_cache){
    %conf=read_config($params_cache);
}


if( !check_installed_pkgs() ){

    print "Not all nessesary packages are installed! Exiting...\n";
    exit;
}

unlink "/etc/squid/.netreg.tag" if $opt_S or $opt_A;
unlink "/etc/dhcpd/.netreg.tag" if $opt_D or $opt_A;
unlink "/etc/httpd/conf/.netreg.tag" if $opt_H or $opt_A;
unlink "/var/www/.netreg.tag" if $opt_W or $opt_A;

misc_config();
print "---=---\n";
squid();
print "---=---\n";
dhcpd();
print "---=---\n";
httpd();
print "---=---\n";
netreg_web_files();

print "---=---\n";
iptables();
print "---=---\n";
set_network();







sub check_installed_pkgs
{
    my $ret_code=1;
    do {print "Couldn't find /usr/sbin/squid. Is Squid installed?\n"; $ret_code=0;} if !-f "/usr/sbin/squid";
    do {print "Couldn't find /usr/sbin/dhcpd. Is DHCPD installed?\n"; $ret_code=0;} if !-f "/usr/sbin/dhcpd";
    do {print "Couldn't find /usr/sbin/httpd. Is HTTPD installed?\n"; $ret_code=0;} if !-f "/usr/sbin/httpd";
    do {print "Couldn't find /sbin/iptables. Is IPTABLES installed?\n"; $ret_code=0;} if !-f "/sbin/iptables";

    return $ret_code;
}

# Squid
sub squid
{

if( !-f "/etc/squid/.netreg.tag"){
# first time
`/etc/init.d/squid stop`;
print "Stopping Squid ...\n" if $debug;
sleep 3;
print "Copying squid.conf and saving original - first time\n" if $debug;
`mv /etc/squid/squid.conf /etc/squid/squid.conf.ORIG`;
`cp $home/squid/squid.conf /etc/squid/squid.conf`;

print "Starting Squid...\n" if $debug;
`/etc/init.d/squid start`;

}
else{
print "NetReg Squid configuration has been installed - doing nothing\n" if $debug;

}
`touch /etc/squid/.netreg.tag`;
}

# DHCP

sub dhcpd
{

    `mkdir /etc/dhcpd` if ! -d "/etc/dhcpd";


if( !-f "/etc/dhcpd/.netreg.tag"){
# first time
print "Stopping DHCPD...\n" if $debug;
sleep 3;
`cp $home/netreg/sbin/dhcpdctl /usr/sbin`;
# just in case to clear out /var/run/dhcpd.pid
`$home/netreg/sbin/dhcpdctl stop`;
`killall   /usr/sbin/dhcpd`; 

## uncomment !!! 
`cp $home/netreg/sbin/dhcpdctl /etc/init.d/dhcpd`;
`chmod 755 /etc/init.d/dhcpd`;
unlink "/etc/rc.d/rc0.d/K38dhcpd"; 
unlink "/etc/rc.d/rc1.d/K38dhcpd";
unlink "/etc/rc.d/rc0.d/S47dhcpd";
unlink "/etc/rc.d/rc0.d/S47dhcpd";

`cd /etc/init.d/; /sbin/chkconfig --add dhcpd`;

`cp $home/netreg/bin/refresh-dhcpdconf /usr/local/bin`;
`cp $home/utils/netreg_block_mac.sh /usr/local/bin`;
#`cp $home/utils/netreg_vars.sh /usr/local/bin`;
`cp $home/utils/get_tc_filter_for_mac.pl /usr/local/bin`;
`touch /var/lib/dhcp/netreg_ether.deny` if !-f "/var/lib/dhcp/netreg_ether.deny"; 


print "Copying dhcpd.conf and saving original - first time\n" if $debug;


# saving files to revert back to saved files - the "preserve" option (-P) 
`cp -f --preserve /etc/dhcpd/dhcpd.conf /etc/dhcpd/dhcpd.conf.ESPBAK 2>/dev/null` if -f "/etc/dhcpd/dhcpd.conf";
`cp -f --preserve /etc/dhcpd/dhcpd.conf.new /etc/dhcpd/dhcpd.conf.new.ESPBAK 2>/dev/null` if -f "/etc/dhcpd/dhcpd.conf.new";
`cp -f --preserve /etc/dhcpd/dhcpd.conf.bak /etc/dhcpd/dhcpd.conf.bak.ESPBAK 2>/dev/null` if -f "/etc/dhcpd/dhcpd.conf.bak" ;
`cp -f --preserve /var/lib/dhcp/dhcpd.leases /var/lib/dhcp/dhcpd.leases.ESPBAK 2>/dev/null` if -f "/var/lib/dhcp/dhcpd.leases";
`cp -r --preserve /var/lib/dhcp/dhcpd.leases.tmp /var/lib/dhcp/dhcpd.leases.tmp.ESPBAK 2>/dev/null` if -f "/var/lib/dhcp/dhcpd.leases.tmp";
`cp -f --preserve /var/lib/dhcp/netreg_ether.deny /var/lib/dhcp/netreg_ether.deny.ESPBAK 2>/dev/null` if -f "/var/lib/dhcp/netreg_ether.deny";



# this is a common part for both
`cp -f $home/dhcpd/dhcpd.conf /etc/dhcpd/dhcpd.conf` if !$opt_P;
`cp -f $home/dhcpd/dhcpd.conf /etc/dhcpd/dhcpd.conf.new` if !$opt_P;
`cp -f $home/dhcpd/dhcpd.conf /etc/dhcpd/dhcpd.conf.bak` if !$opt_P;

`chown -R root:$conf{'httpd_Group'} /etc/dhcpd`;
`chmod 664 /etc/dhcpd/*`;
`touch /var/lib/dhcp/dhcpd.leases` if !-f "/var/lib/dhcp/dhcpd.leases";;

`touch /var/lib/dhcp/dhcpd.leases.tmp` if !-f "/var/lib/dhcp/dhcpd.leases.tmp";
`chown  root:$conf{'httpd_Group'} /var/lib/dhcp/dhcpd.leases.tmp`;
`chmod 664  /var/lib/dhcp/dhcpd.leases.tmp`;

# using this hist directory to store history reports
mkdir "/var/lib/dhcp/hist" if !-d "/var/lib/dhcp/hist";
`touch /var/lib/dhcp/hist/dhcpd.leases.tmp` if !-f "/var/lib/dhcp/hist/dhcpd.leases.tmp";
`chown  root:$conf{'httpd_Group'} /var/lib/dhcp/hist/dhcpd.leases.tmp`;
`chmod 664  /var/lib/dhcp/hist/dhcpd.leases.tmp`;


print "Starting DHCPD...\n" if $debug;
`/etc/init.d/dhcpd start`;

if(!dhcpd_crontab_check()){
    print "\nThere is no  refresh-dhcpdconf entry in crontab!!!\n";
    print "Please run \"crontab -e\" and add this string:\n";
    print "0-59/1 * * * * /usr/local/bin/refresh-dhcpdconf\n";
}
`touch /etc/dhcpd/.netreg.tag`;
}
else{
    print "NetReg DHCP configuration has been already installed - doing nothing \n" if $debug;
}

}


sub dhcpd_crontab_check
{

    open(CRN,"/usr/bin/crontab -l|") || die "Can't open /usr/bin/crontab:$!\n";
    while(<CRN>){
	
	if($_ =~ /refresh-dhcpdconf/){
	    return 1;
	}
    }
    return 0;
}


sub iptables
{

    print "We are about to apply a new iptables ruleset.\n";
    print "The host may loose network connectivity!\n";
    print "You can examing the new ruleset at $home/iptables/firewall.sh\n";
    print "Do you want to proceed? [y|N]:";
    my $answer = <STDIN>;
    if($answer =~ /y/i){
	print "Saving the previous iptables rules in ./iptables.saved\n";
	`/sbin/iptables-save > ./iptables.saved`;
	print "Applying the new iptables ruleset and saving in /etc/sysconfig/iptables ...\n";
	
	`sh $home/iptables/firewall.sh`;
	 `/sbin/service iptables save`;

	open(SYSCTL,"/etc/sysctl.conf") || die "Cant open /etc/sysctl.conf :$!\n";
        @sysctl_conf=<SYSCTL>;
	close SYSCTL;
	$sysctl_conf_content = join(//,@sysctl_conf);
	if( $sysctl_conf_content !~ /NetReg\+VAT/ ){
	    `cat $home/iptables/sysctl.conf >> /etc/sysctl.conf`;
	    print "We have added some routing parameters to /etc/sysctl.conf"; 
	    
	}

    }
}


# apache conf/httpd.conf

sub httpd
{

    my $res;
if( !-f "/etc/httpd/conf/.netreg.tag"){
# first time

`/etc/init.d/httpd stop`;
print "Stopping httpd ...\n" if $debug;

for($i=0;$i<10;$i++){
    
    $res=`/etc/init.d/httpd status`;
    last if $res =~ /stopped/;
    print "+";
    sleep 2;
}

print "Copying httpd.conf and saving original - first time\n" if $debug;
`mv /etc/httpd/conf/httpd.conf /etc/httpd/conf/httpd.conf.ORIG`;
`cp $home/httpd/conf/httpd.conf /etc/httpd/conf/httpd.conf`;

print "Starting httpd...\n" if $debug;
`/etc/init.d/httpd start`;
sleep(3);

# dealing with semaphores left by httpd
$res=`/etc/init.d/httpd status`;
if($res !~ /running/){
my $web_u = " $conf{'httpd_User'}";
open(IPCS,"/usr/bin/ipcs | egrep $web_u|") || die "Cant run ipcs:$!";

while(<IPCS>){
    @a=split(/\s+/); 
    `/usr/bin/ipcrm sem $a[1]`;
}
}


`touch /etc/httpd/conf/.netreg.tag`;
`htpasswd   -b /etc/httpd/htpasswd espadmin esp  >/dev/null 2>&1` if -f "/etc/httpd/htpasswd";
`htpasswd   -c -b /etc/httpd/htpasswd espadmin esp  >/dev/null 2>&1` if !-f "/etc/httpd/htpasswd";


}
else{
print "NetReg httpd configuration has been already installed - doing nothing\n" if $debug;
}

}


# NetReg related cgi-bin/ and htdocs/ 

sub netreg_web_files
{
if( !-f "/var/www/.netreg.tag"){
# first time
print "Copying NetReg web files, the older version of files will be overwritten\n" if $debug;

mkdir "/var/www/htdocs" if !-d  "/var/www/htdocs";
`cp -r $home/netreg/htdocs/* /var/www/htdocs/`;
mkdir "/var/www/cgi-bin/" if !-d "/var/www/cgi-bin/";
`cp -r $home/netreg/cgi-bin/* /var/www/cgi-bin/`;

# removing subversion related dir
`rm -rf /var/www/htdocs/.svn`;
`rm -rf /var/www/cgi-bin/.svn`;

mkdir "/etc/netreg" if !-d "/etc/netreg";

# it has been deleted from the code tree as its code has been incorporated into cgi-bin/scan.pl
# `cp $home/netreg/etc/netreg/scan_request_via_web.pl /etc/netreg/`;

`cp $home/netreg/etc/netreg/subnet.dat /etc/netreg/ `;

`touch /etc/netreg/esp.cfg`;
`chown $conf{'httpd_User'}.$conf{'httpd_Group'} /etc/netreg/esp.cfg `;
`chmod 664 /etc/netreg/esp.cfg`;
#
`touch /var/www/.netreg.tag`;
}
else{
print "NetReg web files have been already installed - doing nothing\n" if $debug;
}

}


sub set_network
{
   my  $res=`/sbin/ifconfig`;
   my ($dev,$mac)=($res =~ /([^\s]+) .*Link.*HWaddr (.+)/);

   print "Setting up new network parameters by coping ifcfg files from $home/net/ to /etc/sysconfig/network-scripts/ \n/usr/sbin/system-config-network-gui utility can be used to change network parameters\n";
   if( -f "$home/net/ifcfg-$dev"){

# removing previous files 
       `rm -f /etc/sysconfig/networking/profiles/default/ifcfg-$dev` if -f "/etc/sysconfig/networking/profiles/default/ifcfg-$dev";
       `rm -f /etc/sysconfig/networking/profiles/default/ifcfg-$dev:1` if -f "/etc/sysconfig/networking/profiles/default/ifcfg-$dev:1";
       `rm -f /etc/sysconfig/networking/devices/ifcfg-$dev` if -f "/etc/sysconfig/networking/devices/ifcfg-$dev";
       `rm -f /etc/sysconfig/networking/devices/ifcfg-$dev:1` if -f "/etc/sysconfig/networking/devices/ifcfg-$dev:1";
       `cp $home/net/ifcfg-$dev /etc/sysconfig/network-scripts/`;
       `cp $home/net/ifcfg-$dev:1 /etc/sysconfig/network-scripts/`;
`/sbin/ifdown $dev; /sbin/ifup $dev`;
   }
}


sub misc_config
{

`echo chmod 755 $home/utils/mac-detect.sh !!!!!!!!!!!!!!`;
`chmod 755 $home/utils/mac-detect.sh`;
`cp $home/utils/mac-detect.sh  /usr/local/bin`;


}



sub help
{
    print <<EOH;
This script copies configuration/data files from $home in needed places
It's interactive. It require a reply for the iptables configuration.
Usage: $0 [-hSDHW]
-h  - help
-S  - overwrire Squid config with files from $home
-D  - overwrite DHCPD config with files from $home
-H  - overwrite HTTPD config with files from $home
-W  - overwrite cgi-bin/ and htdocs/ with files from $home
-A  - overwrite all of the above, works like -S -D -H -W together
-P  - preserver old /etc/dhcpd/dhcpd.conf[.new] /var/lib/dhcp/dhcpd.leases /var/lib/dhcp/netreg_ether.deny

EOH

}
