#!/usr/bin/perl

# this script parses dhcpd.lease 
# see Perl CookBook pp 173-177
# http://www.perl.com/pub/a/1999/10/DBI.html
# modified: June 7/03  

use Time::Local;
use DBI;
use Getopt::Std;

getopts("SDhd");
$db="dhcp";
$host="localhost";
$table="lease";
$user="dbuser";
$password="dw__aPM";
$db_socket="/var/lib/mysql/mysql.sock";
$dsn="DBI:mysql:$db:mysql_socket=$db_socket:host=$host";

if($opt_h){ help(); exit;}
if($opt_S){ sql();exit;}
if($opt_D){ db_create();exit}
$debug=1 if $opt_d;


$dbh = DBI->connect($dsn,$user,$password) or
                   die "Cannot connect to database:$db $!" . DBI->errstr; 
my $sth_in  = $dbh->prepare(qq{
           INSERT INTO $table (ip,ip1,ip2,ip3,ip4,stime,etime,ether) VALUES (?, ?, ?, ?, ?, ?, ?, ?)  }) || die $dbh->errstr;

my $sth_del = $dbh->prepare("DELETE FROM $table") || die $dbh->errstr;

# clean the table
$sth_del->execute() || die $dbh->errstr;



while(<>)
{
 
if( /^lease (.+) \{/ ... /^\}/)
    {
	$indx=$1;
	chomp;
	$t{$indx}=1;
	if(($y,$m,$d,$hh,$mm,$ss)= $_ =~ /starts \d (\d\d\d\d)\/(\d\d)\/(\d\d) (\d\d)\:(\d\d)\:(\d\d)/){
	    $stime=timegm($ss,$mm,$hh,$d,$m-1,$y-1900);
# converting to localtime
# 	    ($ss,$mm,$hh,$d,$m,$y)=localtime($stime);
	    @r=($stime);
	}
	
	if(($y,$m,$d,$hh,$mm,$ss)=$_ =~ /ends \d (\d\d\d\d)\/(\d\d)\/(\d\d) (\d\d)\:(\d\d)\:(\d\d)/){
	    $etime=timegm($ss,$mm,$hh,$d,$m-1,$y-1900);
	    push @r,$etime;
	}
	
	if(($ether)=$_=~/hardware ethernet (.+)\;/){
	    push @r,$ether;
	    to_db(($indx,@r));
	}
   }

elsif (! /^$/)  {
    ;
#    print "we are here!!!!!\n";
#    chomp;
#   $t{'MY_FREELINES'} = [ @{ $t{'MY_FREELINES'} }, $_ ];
}

}

#delete $t{''};
#foreach $k (keys %t){ print "- $k - \n";}


$dbh->disconnect;

# returns doulble word ip, and four octets
sub ip2dword
{
    my $ip= shift;
    my ($ip1,$ip2,$ip3,$ip4)=($ip =~ /(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/);
    $dw_ip= ($ip1 << 24) + ($ip2 << 16) + ($ip3 << 8) + ($ip4);
    return ($dw_ip,$ip1,$ip2,$ip3,$ip4);
}


sub to_db
{
    my ($ip,$stime,$etime,$ether)=@_;
    my ($dw_ip,$ip1,$ip2,$ip3,$ip4)=ip2dword($ip);

print "$dw_ip $ip1 $ip2 $ip3 $ip4 $stime $etime $ether\n" if $debug;

 $sth_in->execute( $dw_ip,$ip1,$ip2,$ip3,$ip4,$stime,$etime,$ether) or die $dbh->errstr;
$sth_in->finish;

}

# print SQL to create a table
sub sql
{

    print <<EOSQL;
CREATE TABLE lease(  
                       ip         INT      UNSIGNED NOT NULL,
                       ip1        TINYINT  UNSIGNED,
                       ip2        TINYINT  UNSIGNED,
                       ip3        TINYINT  UNSIGNED,
                       ip4        TINYINT  UNSIGNED,
                       stime      BIGINT    UNSIGNED,
                       etime      BIGINT    UNSIGNED,
                       ether      CHAR(17),
		     INDEX ip_index (ip,ip1,ip2,ip3,ip4,stime,etime,ether))

EOSQL
}


sub db_create
{
 
    print <<EODB;
Run these commands manually to create/set permissions on/fill the $db database

- mysqladmin -u root -p  create $db
- echo \"grant ALL PRIVILEGES on $db.* to $user\@$host IDENTIFIED BY '$password'\" | mysql -u root -p
- $0 -S | mysql -u $user --password=$password $db
- create ~/.my.cnf and set permissions with chmod 600 ~/.my.cnf
EODB
}

sub help
{
    print <<EOH;
This program fills the $table in the $db database from the dhcpd.lease file
Usage: $0 [-h] [-S] [-D] [-d]
-h    - help
-S    - SQL to create a table
-D    - print database related help
-d    - print some debug info
EOH
}
