#!/bin/sh

# script to block/unblock a mac

TYPE=$1
ADDR=$3
COMMENT=$4
ACLFILE="/var/lib/dhcp/netreg_ether.deny"
IPTABLES="/sbin/iptables"
MYEUID=`id -u`
PERL="/usr/bin/perl"
DHCPLEASES="/var/lib/dhcp/dhcpd.leases"
DHCPDCONF="/etc/dhcpd/dhcpd.conf.new"
LOCAL=/usr/local/bin

# Group ID of the webserver (typicaly : webuser)
# Default router for a client machine : netreg box IP
GROUP=`perl -e '$g=(stat "/etc/dhcpd/dhcpd.conf")[5]; $gn=(getgrgid $g)[0]; print $gn'`
ROUTER=`/sbin/ifconfig eth0 | grep "inet addr:"| cut -d":" -f2 | cut -d" " -f 1`

IFACE0=eth0

# retun an ip for a MAC from dhcpd.leases
function mac_to_ip
{
echo `cat $DHCPLEASES | $PERL -e '
$i=0;
while(<STDIN>)
{
if( /^lease (.+) \{/ ... /^\}/)
    { 
	$ip=$1;
	if(($ether)=$_=~/hardware ethernet (.+)\;/){
# adding index prefix for sorting later to extract the last ip address assigment
		$t{"$i" . "_" ."$ip"}=$ether;
		$i++;
	    }
	 
	    if($_ =~ /ends \d+ (.+)\;/){
		    $times{"$i" . "_" ."$ip"}=$1;
		}
	 


   }
}

# sorting to get the latest IP-MAC pair
for (sort {$b<=>$a}(keys %t)){

($lip,$lends)=();
$return="no ip found";

    if(uc($t{$_}) eq uc($ARGV[0])){
	    $_ =~ /\d+_(.+)/;
	    $lip=$1;
	    $lends=$times{$_};
	    $return = "$lip lease ends $lends";
	    last;
	}
}

print $return;
' $1
`
}

# return the list of blocked MAC addresses
function list_blocked
{

for m in `(cat $ACLFILE | egrep '^[0-9]' | cut -d" " -f 1; $IPTABLES -n -L -t nat | grep -i  "tcp dpt:80 redir ports 80" | $PERL -ne '@f=split; print "$f[6]\n"') | sort -f | uniq -i`
do

LIP=`mac_to_ip $m`

if $IPTABLES -n -L -t nat | grep -q -i  "MAC $m tcp dpt:80 redir ports 80"
then
    echo -n "$m ($LIP): Full block - "
else 
    echo -n "$m ($LIP): Quarantine block - "
fi

echo  `cat $ACLFILE | egrep "^$m "` | cut -d" " -f2-

  
done
}




if [ x$TYPE = x"-L" ]
then
    list_blocked
    $LOCAL/get_tc_filter_for_mac.pl -M all -L
exit
fi


case $# in
"3" | "4" | "5")

[ x$MYEUID != x0 ] && {
echo "Script has to be run as root user"
exit
}

if [ x$1 != x"-F" -a x$1 != x"-Q" -a x$1 != x"-B" -a x$1 != x"-A" ]
then
echo "The block type has to be '-F' for Full block or '-Q' for Quarantine block or -B for Bandwidth Limiting or -A for removing all blocks at once for a particular address"
exit
fi



if `$PERL -e 'if($ARGV[0]=~/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/){exit 0;}else{ exit 1;}' $ADDR`
then

MAC=`cat $DHCPLEASES | $PERL -e '
while(<STDIN>)
{
if( /^lease (.+) \{/ ... /^\}/)
    { 
     $ip=$1;
     
     if(($ether)=$_=~/hardware ethernet (.+)\;/){
     $t{$ip}=$ether;
        }
   }
}
for (keys %t){if($ARGV[0] eq $_) {print uc($t{$_});}
}
' $ADDR
`
if [ x$MAC = x ]
    then
    echo "No MAC address found in $DHCPLEASES"
    exit
else 
    echo "$ADDR MAC $MAC"
fi
	

else
    # normalizing MAC to to upper cased with two digits per byte: XX:YY:XX:ZZ:YY:ZZ
    MAC=`$PERL -e 'print join ":",map {sprintf "%02s",uc($_)} split(/:/,$ARGV[0]);' $ADDR`
 fi
    

case $2 in

"block")

$IPTABLES -t nat -L -n | grep -i $MAC >/dev/null && {
echo "$MAC seems to be already blocked. Exiting..."
exit
}

# changing permissions to block httpd write access, removing entry, changing permissions back
chown root.root $DHCPDCONF

# bandwidth limiting(-B option) changes default gw
if [ $1 = "-B" ]
    then
    sed -i "s/hardware ethernet $MAC ; }/hardware ethernet $MAC ; option routers $ROUTER ; }/" $DHCPDCONF

    tc qdisc show ingress | grep -q "qdisc ingress ffff:" || {

# creating a QDISC if it's not created
	tc qdisc add dev eth0 handle ffff: ingress
    }

# Get first 4 bytes and the next 2 bytes of the MAC address
MAC1=` echo $MAC | perl -ne '@oct=split(/\:/,$_); print @oct[0..3]'`
MAC2=` echo $MAC | perl -ne '@oct=split(/\:/,$_); print @oct[4..5]'`

if [ x$3 != x -a x$4 != x ]
    then
    PKTRATE=$4
    PKTBURST=$5
else
    PKTRATE=3kbit
    PKTBURST=3k
 fi


echo  "$PKTRATE $PKTBURST"

FILTER=`$LOCAL/get_tc_filter_for_mac.pl -M $MAC`

if [ -z "$FILTER" ]
    then

    tc filter add dev $IFACE0 parent ffff: protocol ip prio 50 u32 match u16 0x0800 0xFFFF at -2 match u16 0x$MAC2 0xFFFF at -4 match u32 0x$MAC1 0xFFFFFFFF at -8 police rate $PKTRATE burst $PKTBURST drop  flowid :1

    else
    echo "There is ALREADY a filter rule for $MAC : $FILTER"
fi


else 

    sed -i "/{.*hardware ethernet $MAC.*}/d" $DHCPDCONF

[ $1 = "-F" ] && {
$IPTABLES -t nat -I PREROUTING -p tcp -m mac --mac-source $MAC --dport 80 -j REDIRECT --to-port 80 &&
$IPTABLES -t nat -I PREROUTING -p tcp -m mac --mac-source $MAC --dport 443 -j REDIRECT --to-port 443 
} 

grep  "^$MAC" $ACLFILE >/dev/null || {
    echo $MAC $COMMENT >> $ACLFILE
}

fi

chown root.$GROUP  $DHCPDCONF

;;

"unblock")

# bandwidth limiting(-B option) changes default gw
if [ $1 = "-B" ]
then

    $LOCAL/get_tc_filter_for_mac.pl -M all -X $MAC > rerun.sh

    tc qdisc del dev eth0 handle ffff: ingress
    tc qdisc add dev eth0 handle ffff: ingress
    sh rerun.sh
    rm rerun.sh

    sed -i "s/hardware ethernet $MAC ; option routers $ROUTER ; }/hardware ethernet $MAC ; }/" $DHCPDCONF
chown root.$GROUP  $DHCPDCONF

else

    [ $1 = "-F" ] && {
	$IPTABLES -t nat -D PREROUTING -p tcp -m mac --mac-source $MAC --dport 80 -j REDIRECT --to-port 80
	$IPTABLES -t nat -D PREROUTING -p tcp -m mac --mac-source $MAC --dport 443 -j REDIRECT --to-port 443
    }

# this is used for -Q & -F unblocking as -Q is put ip into the file only
sed -i "/$MAC/d" $ACLFILE >/dev/null

fi


;;

unblockall)

# removing all blocks for an address
# without analysing if they exists, errors're sent to /dev/null

    $LOCAL/get_tc_filter_for_mac.pl -M all -X $MAC > rerun.sh

    tc qdisc del dev eth0 handle ffff: ingress
    tc qdisc add dev eth0 handle ffff: ingress
    sh rerun.sh
    rm rerun.sh

    sed -i "s/hardware ethernet $MAC ; option routers $ROUTER ; }/hardware ethernet $MAC ; }/" $DHCPDCONF
chown root.$GROUP  $DHCPDCONF

    $IPTABLES -t nat -D PREROUTING -p tcp -m mac --mac-source $MAC --dport 80 -j REDIRECT --to-port 80 > /dev/null 2>&1
    $IPTABLES -t nat -D PREROUTING -p tcp -m mac --mac-source $MAC --dport 443 -j REDIRECT --to-port 443 > /dev/null 2>&1
    
    sed -i "/$MAC/d" $ACLFILE >/dev/null

;;


*) 
	echo 'Wrong command, it has to be "block" or "unblock"'
	exit
	;;
esac
;;

*)
	echo "Script adds a MAC address to be blocked by NetReg"
	echo "There are two types of blocking: Full (only NetReg web is accessable) and Quarantine (No Registration available)"
	echo "Usage: $0 <-L|-F|-Q> <block|unblock> ADDRESS 'comment' | -A unblockall ADDR | -B <block ADDR pkt_rate pkt_burst|unblock ADDR"
	echo "-L    - list all blocks"
	echo "-F    - Full block with packet filtering"
	echo "-Q    - Quarantine block without packet filtering,the Registration is not available"
	echo "-B    - Bandwidth limiting, traffic will be put into a smaller pipe" 
	echo "block|unblock - a command"
	echo
	echo "ADDRESS - MAC address or IP address, the MAC for IP will be taken from $DHCPLEASES"
	echo "'comment'     - the comment to be put into netreg_ether.deny file, please use quotes"
        echo "Ex: $0 -F block 00:aa:bb:cc:01:20 'This host is infected with X worm'"
	echo "     The host will only be able to connect to the NetReg web server"
        echo "Ex: $0 -F unblock 00:aa:bb:cc:01:20 'This host is infected with X worm'"
        echo "Ex: $0 -Q block 00:aa:bb:cc:01:20 'The host traffic is suspicious '"
	echo "     The last command doen't restrict access to Microsoft patches "
	echo "Ex: $0 -B block 00:aa:bb:cc:01:20"
	echo "     The traffi will be shaped"
	echo
	echo "-A unblockall ADDRESS - remove All blocks for this address" 
	

        ;;
esac


/sbin/iptables-save  > /etc/sysconfig/iptables



